@@ -1,6 +1,6 @@
 /* airpcap_loader.c
  *
- * $Id: airpcap_loader.c 23623 2007-11-27 18:52:51Z guy $
+ * $Id: airpcap_loader.c 25248 2008-05-07 05:26:40Z etxrab $
  *
  * Giorgio Tino <giorgio.tino@cacetech.com>
  * Copyright (c) CACE Technologies, LLC 2006
@@ -54,9 +54,6 @@
 #include <airpcap.h>
 #include "airpcap_loader.h"
 
-#ifdef NEED_G_ASCII_STRCASECMP_H
-#include "g_ascii_strcasecmp.h"
-#endif
 
 /*
  * We load dynamically the dag library in order link it only when
@@ -2542,7 +2539,7 @@ get_runtime_airpcap_version(GString *str)
     }
 
     g_PAirpcapGetVersion(&vmaj, &vmin, &vrev, &build);
-    g_string_sprintfa(str, "with AirPcap %d.%d.%d build %d", vmaj, vmin,
+    g_string_append_printf(str, "with AirPcap %d.%d.%d build %d", vmaj, vmin,
 	vrev, build);
 }
 #endif /* HAVE_AIRPCAP */
