@@ -2,7 +2,7 @@
  * Routines for X.518 (X.500 Distributed Operations)  packet dissection
  * Graeme Lunt 2005
  *
- * $Id: packet-dsp-template.c 23324 2007-10-31 21:24:16Z etxrab $
+ * $Id: packet-dsp-template.c 24987 2008-04-13 16:21:08Z gal $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -303,11 +303,7 @@ void proto_register_dsp(void) {
 
   /* Register our configuration options for DSP, particularly our port */
 
-#ifdef PREFERENCE_GROUPING
   dsp_module = prefs_register_protocol_subtree("OSI/X.500", proto_dsp, prefs_register_dsp);
-#else
-  dsp_module = prefs_register_protocol(proto_dsp, prefs_register_dsp);
-#endif
 
   prefs_register_uint_preference(dsp_module, "tcp.port", "DSP TCP Port",
 				 "Set the port for DSP operations (if other"
