@@ -2,7 +2,7 @@
  * Routines for h245 packet dissection
  * Copyright 2004, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-h245-template.c 24150 2008-01-21 06:28:23Z etxrab $
+ * $Id: packet-h245-template.c 25164 2008-04-24 06:36:42Z kukosa $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -39,6 +39,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/conversation.h>
+#include <epan/strutil.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -327,43 +328,40 @@ static void h245_setup_channels(packet_info *pinfo, channel_info_t *upcoming_cha
 {
 	gint *key;
 	GHashTable *rtp_dyn_payload = NULL;
+	struct srtp_info *dummy_srtp_info = NULL;
 
 	if (!upcoming_channel) return;
 
+	/* T.38 */
 	if (!strcmp(upcoming_channel->data_type_str, "t38fax")) {
 		if (upcoming_channel->media_addr.addr.type!=AT_NONE && upcoming_channel->media_addr.port!=0 && t38_handle) {
 			t38_add_address(pinfo, &upcoming_channel->media_addr.addr, 
 							upcoming_channel->media_addr.port, 0, 
 							"H245", pinfo->fd->num);
 		}
-	} else {
-		if (upcoming_channel->rfc2198 > 0) {
-#if GLIB_MAJOR_VERSION < 2
-			rtp_dyn_payload = g_hash_table_new(g_int_hash, g_int_equal);
-#else
-			rtp_dyn_payload = g_hash_table_new_full(g_int_hash, g_int_equal, g_free, g_free);
-#endif
-			key = g_malloc(sizeof(gint));
-			*key = upcoming_channel->rfc2198;
-			g_hash_table_insert(rtp_dyn_payload, key, g_strdup("red"));
-		}
-		if (upcoming_channel->media_addr.addr.type!=AT_NONE && upcoming_channel->media_addr.port!=0 && rtp_handle) {
-			if (upcoming_channel->srtp_flag) {
-				struct srtp_info *dummy_srtp_info = se_alloc0(sizeof(struct srtp_info));
-				srtp_add_address(pinfo, &upcoming_channel->media_addr.addr, 
-								upcoming_channel->media_addr.port, 0, 
-								"H245", pinfo->fd->num, rtp_dyn_payload, dummy_srtp_info);
-			} else {
-				rtp_add_address(pinfo, &upcoming_channel->media_addr.addr, 
-								upcoming_channel->media_addr.port, 0, 
-								"H245", pinfo->fd->num, rtp_dyn_payload);
-			}
-		}
-		if (upcoming_channel->media_control_addr.addr.type!=AT_NONE && upcoming_channel->media_control_addr.port!=0 && rtcp_handle) {
-			rtcp_add_address(pinfo, &upcoming_channel->media_control_addr.addr, 
-							upcoming_channel->media_control_addr.port, 0, 
-							"H245", pinfo->fd->num);
-		}
+		return;
+	}
+
+	/* (S)RTP, (S)RTCP */
+	if (upcoming_channel->rfc2198 > 0) {
+		rtp_dyn_payload = g_hash_table_new_full(g_int_hash, g_int_equal, g_free, g_free);
+		key = g_malloc(sizeof(gint));
+		*key = upcoming_channel->rfc2198;
+		g_hash_table_insert(rtp_dyn_payload, key, g_strdup("red"));
+	}
+
+	if (upcoming_channel->srtp_flag) {
+		dummy_srtp_info = se_alloc0(sizeof(struct srtp_info));
+	}
+	if (upcoming_channel->media_addr.addr.type!=AT_NONE && upcoming_channel->media_addr.port!=0 && rtp_handle) {
+		srtp_add_address(pinfo, &upcoming_channel->media_addr.addr, 
+						upcoming_channel->media_addr.port, 0, 
+						"H245", pinfo->fd->num, rtp_dyn_payload, dummy_srtp_info);
+	}
+	if (upcoming_channel->media_control_addr.addr.type!=AT_NONE && upcoming_channel->media_control_addr.port!=0 && rtcp_handle) {
+		srtcp_add_address(pinfo, &upcoming_channel->media_control_addr.addr, 
+						upcoming_channel->media_control_addr.port, 0, 
+						"H245", pinfo->fd->num, dummy_srtp_info);
 	}
 }
 
@@ -445,7 +443,7 @@ dissect_h245_FastStart_OLC(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tr
 	  h245_pi->msg_type = H245_OpenLogChn;
 
   if (codec_str && codec_type){
-        strncpy(codec_str, codec_type, 50);
+        g_strlcpy(codec_str, codec_type, 50);
   }
 
 }
@@ -561,7 +559,7 @@ void proto_reg_handoff_h245(void) {
 	t38_handle = find_dissector("t38");
 	data_handle = find_dissector("data");
 	h263_handle = find_dissector("h263data");
-	amr_handle = find_dissector("amr_if2");
+	amr_handle = find_dissector("amr_if2_nb");
 
 
 	h245_handle=create_dissector_handle(dissect_h245, proto_h245);
