@@ -15,9 +15,9 @@
  * See RFC 3414 for User-based Security Model for SNMPv3
  * See RFC 3826 for  (AES) Cipher Algorithm in the SNMP USM
  * See RFC 2578 for Structure of Management Information Version 2 (SMIv2)
- * Copyright (C) 2007 Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
+ * Copyright (C) 2007 Luis E. Garcia Ontanon <luis@ontanon.org>
  *
- * $Id: packet-snmp-template.c 26913 2008-12-04 20:07:09Z gerald $
+ * $Id: packet-snmp-template.c 26058 2008-08-22 14:28:03Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -615,7 +615,6 @@ show_oid_index:
 										break;
 									case OID_KEY_TYPE_IMPLIED_STRING:
 									case OID_KEY_TYPE_IMPLIED_BYTES:
-									case OID_KEY_TYPE_ETHER:
 										suboid = &(subids[key_start]);
 										buf_len = key_len;
 										break;
@@ -653,9 +652,6 @@ show_oid_index:
 									case OID_KEY_TYPE_IMPLIED_BYTES:
 										proto_tree_add_bytes(pt_name,k->hfid,tvb,name_offset,buf_len, buf);
 										break;
-									case OID_KEY_TYPE_ETHER:
-										proto_tree_add_ether(pt_name,k->hfid,tvb,name_offset,buf_len, buf);
-										break;
 									case OID_KEY_TYPE_IPADDR: {
 										guint32* ipv4_p = (void*)buf;
 										proto_tree_add_ipv4(pt_name,k->hfid,tvb,name_offset,buf_len, *ipv4_p);
@@ -1124,7 +1120,7 @@ static gboolean snmp_usm_auth_md5(snmp_usm_params_t* p, guint8** calc_auth_p, gu
 	guint8* auth;
 	guint8* key;
 	guint key_len;
-	guint8 *calc_auth;
+	guint8 calc_auth[16];
 	guint start;
 	guint end;
 	guint i;
@@ -1164,8 +1160,6 @@ static gboolean snmp_usm_auth_md5(snmp_usm_params_t* p, guint8** calc_auth_p, gu
 		msg[i] = '\0';
 	}
 
-	calc_auth = ep_alloc(16);
-
 	md5_hmac(msg, msg_len, key, key_len, calc_auth);
 
 	if (calc_auth_p) *calc_auth_p = calc_auth;
@@ -1182,7 +1176,7 @@ static gboolean snmp_usm_auth_sha1(snmp_usm_params_t* p _U_, guint8** calc_auth_
 	guint8* auth;
 	guint8* key;
 	guint key_len;
-	guint8 *calc_auth;
+	guint8 calc_auth[20];
 	guint start;
 	guint end;
 	guint i;
@@ -1222,8 +1216,6 @@ static gboolean snmp_usm_auth_sha1(snmp_usm_params_t* p _U_, guint8** calc_auth_
 		msg[i] = '\0';
 	}
 
-	calc_auth = ep_alloc(20);
-
 	sha1_hmac(key, key_len, msg, msg_len, calc_auth);
 
 	if (calc_auth_p) *calc_auth_p = calc_auth;
@@ -1827,8 +1819,6 @@ static void snmp_users_update_cb(void* p _U_, const char** err) {
 	*err = NULL;
 
 	if (! ue->user.userName.len) g_string_append(es,"no userName, ");
-	if (ue->user.authPassword.len < 8) g_string_sprintfa(es,"short authPassword (%d), ", ue->user.authPassword.len);
-	if (ue->user.privPassword.len < 8) g_string_sprintfa(es,"short privPassword (%d), ", ue->user.privPassword.len);
 
 	if (es->len) {
 		g_string_truncate(es,es->len-2);
