@@ -3,7 +3,7 @@
  * Copyright 2004 - 2005, Tim Endean <endeant@hotmail.com>
  * Built from the gsm-map dissector Copyright 2004 - 2005, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-tcap-template.c 23154 2007-10-11 22:49:52Z morriss $
+ * $Id: packet-tcap-template.c 25080 2008-04-16 14:28:22Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -36,6 +36,7 @@
 #include <epan/oids.h>
 #include <epan/emem.h>
 #include <epan/asn1.h>
+#include <epan/strutil.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -99,8 +100,6 @@ static proto_tree * tcap_stat_tree=NULL;
 static dissector_handle_t data_handle;
 static dissector_handle_t ansi_tcap_handle;
 
-static dissector_table_t sccp_ssn_table;
-
 static void raz_tcap_private(struct tcap_private_t * p_tcap_private);
 static int dissect_tcap_param(asn1_ctx_t *actx, proto_tree *tree, tvbuff_t *tvb, int offset);
 static int dissect_tcap_UserInformation(gboolean implicit_tag _U_, tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index _U_);
@@ -238,7 +237,7 @@ dissect_tcap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		if ( p_tcap_context && cur_oid && !p_tcap_context->oid_present ) {
 			/* Save the application context and the sub dissector */
 			ber_oid_dissector_table = find_dissector_table("ber.oid");
-			strncpy(p_tcap_context->oid,cur_oid, LENGTH_OID);
+			g_strlcpy(p_tcap_context->oid,cur_oid, LENGTH_OID);
 			p_tcap_context->oid_present=TRUE;
 			if ( (subdissector_handle = dissector_get_string_handle(ber_oid_dissector_table, cur_oid)) ) {
 				p_tcap_context->subdissector_handle=subdissector_handle;
@@ -256,15 +255,8 @@ void
 proto_reg_handoff_tcap(void)
 {
 
-    static gboolean prefs_initialized = FALSE;
-
-    if (! prefs_initialized) {
-        sccp_ssn_table = find_dissector_table("sccp.ssn");
-        prefs_initialized = TRUE;
-    }
-
     data_handle = find_dissector("data");
-	ansi_tcap_handle = find_dissector("ansi_tcap");
+    ansi_tcap_handle = find_dissector("ansi_tcap");
 
 #include "packet-tcap-dis-tab.c"
 }
@@ -356,7 +348,7 @@ proto_register_tcap(void)
     proto_register_field_array(proto_tcap, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
 
-    tcap_module = prefs_register_protocol(proto_tcap, proto_reg_handoff_tcap);
+    tcap_module = prefs_register_protocol(proto_tcap, NULL);
 
 #if 0
     prefs_register_enum_preference(tcap_module, "standard", "ITU TCAP standard",
@@ -565,7 +557,7 @@ dissect_tcap_ITU_ComponentPDU(gboolean implicit_tag _U_, tvbuff_t *tvb, int offs
 				  /* ACN, changed, Fallback to lower version
 				   * and update the subdissector (purely formal)
 				   */
-				  strncpy(p_tcap_context->oid,cur_oid, LENGTH_OID);
+				  g_strlcpy(p_tcap_context->oid,cur_oid, LENGTH_OID);
 				  if ( (subdissector_handle = dissector_get_string_handle(ber_oid_dissector_table, cur_oid)) ) {
 					  p_tcap_context->subdissector_handle=subdissector_handle;
 					  p_tcap_context->subdissector_present=TRUE;
@@ -573,7 +565,7 @@ dissect_tcap_ITU_ComponentPDU(gboolean implicit_tag _U_, tvbuff_t *tvb, int offs
 			  }
 		  } else {
 			  /* We do not have the OID in the TCAP context, so store it */
-			  strncpy(p_tcap_context->oid,cur_oid, LENGTH_OID);
+			  g_strlcpy(p_tcap_context->oid,cur_oid, LENGTH_OID);
 			  p_tcap_context->oid_present=TRUE;
 			  /* Try to find a subdissector according to OID */
 			  if ( (subdissector_handle
