@@ -2,7 +2,7 @@
 #
 # Run this to generate all the initial makefiles.
 #
-# $Id: autogen.sh 23796 2007-12-07 22:33:46Z guy $
+# $Id: autogen.sh 25242 2008-05-05 22:01:34Z guy $
 
 DIE=true
 PROJECT="Wireshark"
@@ -47,14 +47,14 @@ esac
 
 AMVER=`$AUTOMAKE --version | grep '^automake' | sed 's/.*) *//'`
 case "$AMVER" in
-1.[6-9]* | 1.[1][0-9]*)
+1.9* | 1.[1][0-9]*)
   ;;
 
 *)
 
   cat >&2 <<_EOF_
 
-	You must have automake 1.6 or later installed to compile $PROJECT.
+	You must have automake 1.9 or later installed to compile $PROJECT.
 	Download the appropriate package for your distribution/OS,
 	or get the source tarball at ftp://ftp.gnu.org/pub/gnu/automake/
 _EOF_
