@@ -1,7 +1,7 @@
 /* capture_opts.c
  * Routines for capture options setting
  *
- * $Id: capture_opts.c 24446 2008-02-23 19:59:38Z wmeier $
+ * $Id: capture_opts.c 25566 2008-06-23 21:58:16Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -69,20 +69,15 @@
 #include "cmdarg_err.h"
 
 #include "capture-pcap-util.h"
-#include <wiretap/file_util.h>
-
-typedef struct {
-    char *name;
-    pcap_t *pch;
-} if_stat_t;
+#include <wsutil/file_util.h>
 
 static gboolean capture_opts_output_to_pipe(const char *save_file, gboolean *is_pipe);
 
 
 void
-capture_opts_init(capture_options *capture_opts, void *cfile)
+capture_opts_init(capture_options *capture_opts, void *cf)
 {
-  capture_opts->cf                      = cfile;
+  capture_opts->cf                      = cf;
   capture_opts->cfilter                 = g_strdup("");     /* No capture filter string specified */
   capture_opts->iface                   = NULL;             /* Default is "pick the first interface" */
   capture_opts->iface_descr             = NULL;
@@ -153,7 +148,7 @@ capture_opts_log(const char *log_domain, GLogLevelFlags log_level, capture_optio
     g_log(log_domain, log_level, "Filter             : %s", capture_opts->cfilter);
     g_log(log_domain, log_level, "Interface          : %s", capture_opts->iface);
     /* iface_descr may not been filled in and some C Libraries hate a null ptr for %s */
-    g_log(log_domain, log_level, "Interface Descr    : %s", 
+    g_log(log_domain, log_level, "Interface Descr    : %s",
 	  capture_opts->iface_descr ? capture_opts->iface_descr : "<null>");
 #ifdef HAVE_PCAP_REMOTE
     g_log(log_domain, log_level, "Capture source     : %s",
@@ -532,7 +527,7 @@ capture_opts_add_opt(capture_options *capture_opts, int opt, const char *optarg,
     case 'w':        /* Write to capture file x */
         capture_opts->saving_to_file = TRUE;
         g_free(capture_opts->save_file);
-#if defined _WIN32 && (GLIB_MAJOR_VERSION > 2 || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION >= 6))
+#if defined _WIN32 && GLIB_CHECK_VERSION(2,6,0)
         /* since GLib 2.6, we need to convert filenames to utf8 for Win32 */
         capture_opts->save_file = g_locale_to_utf8(optarg, -1, NULL, NULL, NULL);
 #else
@@ -705,90 +700,6 @@ capture_opts_list_interfaces(gboolean machine_readable)
     return 0;
 }
 
-/* Print the number of packets captured for each interface until we're killed. */
-int
-capture_opts_print_statistics(gboolean machine_readable)
-{
-    GList       *if_list, *if_entry, *stat_list = NULL, *stat_entry;
-    if_info_t   *if_info;
-    if_stat_t   *if_stat;
-    int         err;
-    gchar       *err_str;
-    pcap_t      *pch;
-    char        errbuf[PCAP_ERRBUF_SIZE];
-    struct pcap_stat ps;
-
-    if_list = get_interface_list(&err, &err_str);
-    if (if_list == NULL) {
-        switch (err) {
-        case CANT_GET_INTERFACE_LIST:
-            cmdarg_err("%s", err_str);
-            g_free(err_str);
-        break;
-
-        case NO_INTERFACES_FOUND:
-            cmdarg_err("There are no interfaces on which a capture can be done");
-        break;
-        }
-        return err;
-    }
-
-    for (if_entry = g_list_first(if_list); if_entry != NULL; if_entry = g_list_next(if_entry)) {
-        if_info = if_entry->data;
-        pch = pcap_open_live(if_info->name, MIN_PACKET_SIZE, 0, 0, errbuf);
-
-        if (pch) {
-            if_stat = g_malloc(sizeof(if_stat_t));
-            if_stat->name = g_strdup(if_info->name);
-            if_stat->pch = pch;
-            stat_list = g_list_append(stat_list, if_stat);
-        }
-    }
-
-    if (!stat_list) {
-        cmdarg_err("There are no interfaces on which a capture can be done");
-        return 2;
-    }
-
-    if (!machine_readable) {
-        printf("%-15s  %10s  %10s\n", "Interface", "Received",
-            "Dropped");
-    }
-
-    while (1) {    /* XXX - Add signal handling? */
-        for (stat_entry = g_list_first(stat_list); stat_entry != NULL; stat_entry = g_list_next(stat_entry)) {
-            if_stat = stat_entry->data;
-            pcap_stats(if_stat->pch, &ps);
-
-            if (!machine_readable) {
-                printf("%-15s  %10d  %10d\n", if_stat->name,
-                    ps.ps_recv, ps.ps_drop);
-            } else {
-                printf("%s\t%d\t%d\n", if_stat->name,
-                    ps.ps_recv, ps.ps_drop);
-                fflush(stdout);
-            }
-        }
-#ifdef _WIN32
-        Sleep(1 * 1000);
-#else
-        sleep(1);
-#endif
-    }
-
-    /* XXX - Not reached.  Should we look for 'q' in stdin? */
-    for (stat_entry = g_list_first(stat_list); stat_entry != NULL; stat_entry = g_list_next(stat_entry)) {
-        if_stat = stat_entry->data;
-        pcap_close(if_stat->pch);
-        g_free(if_stat->name);
-        g_free(if_stat);
-    }
-    g_list_free(stat_list);
-    free_interface_list(if_list);
-
-    return 0;
-}
-
 
 void capture_opts_trim_snaplen(capture_options *capture_opts, int snaplen_min)
 {
@@ -873,7 +784,7 @@ static int capture_opts_test_for_fifo(const char *path)
 {
 	struct stat statb;
 
-	if (eth_stat(path, &statb) < 0)
+	if (ws_stat(path, &statb) < 0)
 		return errno;
 
 	if (S_ISFIFO(statb.st_mode))
