@@ -1,6 +1,6 @@
 /* dumpcap.c
  *
- * $Id: dumpcap.c 29905 2009-09-14 18:53:27Z gerald $
+ * $Id: dumpcap.c 26037 2008-08-19 05:10:16Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -79,10 +79,10 @@
 #endif
 
 #ifdef _WIN32
-#include "epan/unicode-utils.h"
+#include <wsutil/unicode-utils.h>
 #endif
 
-#include "epan/privileges.h"
+#include <wsutil/privileges.h>
 
 #include "sync_pipe.h"
 
@@ -94,7 +94,7 @@
 
 #include "tempfile.h"
 #include "log.h"
-#include "file_util.h"
+#include "wsutil/file_util.h"
 
 /*
  * Get information about libpcap format from "wiretap/libpcap.h".
@@ -129,7 +129,7 @@ static void capture_loop_stop(void);
  * is interrupted by a signal on UN*X, just go back and try again to
  * read again.
  *
- * On UN*X, we catch SIGINT as a "stop capturing" signal, and, in
+ * On UN*X, we catch SIGUSR1 as a "stop capturing" signal, and, in
  * the signal handler, set a flag to stop capturing; however, without
  * a guarantee of that sort, we can't guarantee that we'll stop capturing
  * if the read will be retried and won't time out if no packets arrive.
@@ -152,7 +152,7 @@ static void capture_loop_stop(void);
  * exit pcap_dispatch() with an indication that no packets have arrived,
  * and will break out of the capture loop at that point.
  *
- * On Windows, we can't send a SIGINT to stop capturing, so none of this
+ * On Windows, we can't send a SIGUSR1 to stop capturing, so none of this
  * applies in any case.
  *
  * XXX - the various BSDs appear to define BSD in <sys/param.h>; we don't
@@ -226,10 +226,10 @@ static const char please_report[] =
     "(This is not a crash; please do not report it as such.)";
 
 /*
- * This needs to be static, so that the SIGINT handler can clear the "go"
+ * This needs to be static, so that the SIGUSR1 handler can clear the "go"
  * flag.
  */
-static loop_data   ld;
+static loop_data   global_ld;
 
 
 /*
@@ -244,7 +244,6 @@ console_log_handler(const char *log_domain, GLogLevelFlags log_level,
 
 /* capture related options */
 static capture_options global_capture_opts;
-static capture_options *capture_opts = &global_capture_opts;
 
 static void capture_loop_packet_cb(u_char *user, const struct pcap_pkthdr *phdr,
   const u_char *pd);
@@ -260,7 +259,7 @@ static void exit_main(int err);
 
 static void report_new_capture_file(const char *filename);
 static void report_packet_count(int packet_count);
-static void report_packet_drops(int drops);
+static void report_packet_drops(guint32 drops);
 static void report_capture_error(const char *error_msg, const char *secondary_error_msg);
 static void report_cfilter_error(const char *cfilter, const char *errmsg);
 
@@ -395,10 +394,104 @@ cmdarg_err_cont(const char *fmt, ...)
   }
 }
 
+typedef struct {
+    char *name;
+    pcap_t *pch;
+} if_stat_t;
+
+/* Print the number of packets captured for each interface until we're killed. */
+static int
+print_statistics_loop(gboolean machine_readable)
+{
+    GList       *if_list, *if_entry, *stat_list = NULL, *stat_entry;
+    if_info_t   *if_info;
+    if_stat_t   *if_stat;
+    int         err;
+    gchar       *err_str;
+    pcap_t      *pch;
+    char        errbuf[PCAP_ERRBUF_SIZE];
+    struct pcap_stat ps;
+
+    if_list = get_interface_list(&err, &err_str);
+    if (if_list == NULL) {
+        switch (err) {
+        case CANT_GET_INTERFACE_LIST:
+            cmdarg_err("%s", err_str);
+            g_free(err_str);
+            break;
+
+        case NO_INTERFACES_FOUND:
+            cmdarg_err("There are no interfaces on which a capture can be done");
+            break;
+        }
+        return err;
+    }
+
+    for (if_entry = g_list_first(if_list); if_entry != NULL; if_entry = g_list_next(if_entry)) {
+        if_info = if_entry->data;
+#ifdef HAVE_PCAP_OPEN
+        pch = pcap_open(if_info->name, MIN_PACKET_SIZE, 0, 0, NULL, errbuf);
+#else
+        pch = pcap_open_live(if_info->name, MIN_PACKET_SIZE, 0, 0, errbuf);
+#endif
+
+        if (pch) {
+            if_stat = g_malloc(sizeof(if_stat_t));
+            if_stat->name = g_strdup(if_info->name);
+            if_stat->pch = pch;
+            stat_list = g_list_append(stat_list, if_stat);
+        }
+    }
+
+    if (!stat_list) {
+        cmdarg_err("There are no interfaces on which a capture can be done");
+        return 2;
+    }
+
+    if (!machine_readable) {
+        printf("%-15s  %10s  %10s\n", "Interface", "Received",
+            "Dropped");
+    }
+
+    global_ld.go = TRUE;
+    while (global_ld.go) {
+        for (stat_entry = g_list_first(stat_list); stat_entry != NULL; stat_entry = g_list_next(stat_entry)) {
+            if_stat = stat_entry->data;
+            pcap_stats(if_stat->pch, &ps);
+
+            if (!machine_readable) {
+                printf("%-15s  %10u  %10u\n", if_stat->name,
+                    ps.ps_recv, ps.ps_drop);
+            } else {
+                printf("%s\t%u\t%u\n", if_stat->name,
+                    ps.ps_recv, ps.ps_drop);
+                fflush(stdout);
+            }
+        }
+#ifdef _WIN32
+        Sleep(1 * 1000);
+#else
+        sleep(1);
+#endif
+    }
+
+    /* XXX - Not reached.  Should we look for 'q' in stdin? */
+    for (stat_entry = g_list_first(stat_list); stat_entry != NULL; stat_entry = g_list_next(stat_entry)) {
+        if_stat = stat_entry->data;
+        pcap_close(if_stat->pch);
+        g_free(if_stat->name);
+        g_free(if_stat);
+    }
+    g_list_free(stat_list);
+    free_interface_list(if_list);
+
+    return 0;
+}
+
 
 #ifdef _WIN32
 static BOOL WINAPI
-capture_cleanup_handler(DWORD dwCtrlType)
+capture_cleanup(DWORD dwCtrlType)
 {
     /* CTRL_C_EVENT is sort of like SIGINT, CTRL_BREAK_EVENT is unique to
        Windows, CTRL_CLOSE_EVENT is sort of like SIGHUP, CTRL_LOGOFF_EVENT
@@ -430,16 +523,16 @@ capture_cleanup_handler(DWORD dwCtrlType)
 }
 #else
 static void
-capture_cleanup_handler(int signum _U_)
+capture_cleanup(int signum)
 {
     /* On UN*X, we cleanly shut down the capture on SIGINT, SIGHUP, and
        SIGTERM.  We assume that if the user wanted it to keep running
        after they logged out, they'd have nohupped it. */
 
-    /* Note: don't call g_log() in the signal handler: if we happened to be in
-     * g_log() in process context when the signal came in, g_log will detect
-     * the "recursion" and abort.
-     */
+    g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO,
+        "Console: Signal");
+    g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
+        "Console: Signal, signal value: %u", signum);
 
     capture_loop_stop();
 }
@@ -689,7 +782,7 @@ cap_pipe_open_live(char *pipename, struct pcap_hdr *hdr, loop_data *ld,
 #endif  /* _WIN32 */
   } else {
 #ifndef _WIN32
-    if (eth_stat(pipename, &pipe_stat) < 0) {
+    if (ws_stat(pipename, &pipe_stat) < 0) {
       if (errno == ENOENT || errno == ENOTDIR)
         ld->cap_pipe_err = PIPNEXIST;
       else {
@@ -716,7 +809,7 @@ cap_pipe_open_live(char *pipename, struct pcap_hdr *hdr, loop_data *ld,
       }
       return -1;
     }
-    fd = eth_open(pipename, O_RDONLY | O_NONBLOCK, 0000 /* no creation so don't matter */);
+    fd = ws_open(pipename, O_RDONLY | O_NONBLOCK, 0000 /* no creation so don't matter */);
     if (fd == -1) {
       g_snprintf(errmsg, errmsgl,
           "The capture session could not be initiated "
@@ -891,7 +984,7 @@ cap_pipe_open_live(char *pipename, struct pcap_hdr *hdr, loop_data *ld,
 error:
   g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG, "cap_pipe_open_live: error %s", errmsg);
   ld->cap_pipe_err = PIPERR;
-  eth_close(fd);
+  ws_close(fd);
   return -1;
 
 }
@@ -1230,19 +1323,8 @@ capture_loop_open_input(capture_options *capture_opts, loop_data *ld,
 "\n"
 "Help can be found at:\n"
 "\n"
-"       http://wiki.wireshark.org/CaptureSetup\n"
-"\n"
-"64-bit Windows:\n"
-"WinPcap does not support 64-bit Windows; you will have to use some other\n"
-"tool to capture traffic, such as netcap.\n"
-"For netcap details see: http://support.microsoft.com/?id=310875\n"
-"\n"
-"Modem (PPP/WAN):\n"
-"Note that version 3.0 of WinPcap, and earlier versions of WinPcap, don't\n"
-"support capturing on PPP/WAN interfaces on Windows NT 4.0 / 2000 / XP /\n"
-"Server 2003.\n"
-"WinPcap 3.1 has support for it on Windows 2000 / XP / Server 2003, but has no\n"
-"support for it on Windows NT 4.0 or Windows Vista (Beta 1).",
+"       http://wiki.wireshark.org/WinPcap\n"
+"       http://wiki.wireshark.org/CaptureSetup\n",
     capture_opts->iface);
 #endif /* _WIN32 */
       }
@@ -1288,7 +1370,7 @@ static void capture_loop_close_input(loop_data *ld) {
   /* if open, close the capture pipe "input file" */
   if (ld->cap_pipe_fd >= 0) {
     g_assert(ld->from_cap_pipe);
-    eth_close(ld->cap_pipe_fd);
+    ws_close(ld->cap_pipe_fd);
     ld->cap_pipe_fd = 0;
   }
 
@@ -1644,7 +1726,7 @@ capture_loop_open_output(capture_options *capture_opts, int *save_file_fd,
         }
       } else {
         /* Try to open/create the specified file for use as a capture buffer. */
-        *save_file_fd = eth_open(capfile_name, O_RDWR|O_BINARY|O_TRUNC|O_CREAT,
+        *save_file_fd = ws_open(capfile_name, O_RDWR|O_BINARY|O_TRUNC|O_CREAT,
                              0600);
       }
     }
@@ -1690,6 +1772,13 @@ capture_loop_open_output(capture_options *capture_opts, int *save_file_fd,
 }
 
 
+static void
+capture_loop_stop_signal_handler(int signo _U_)
+{
+  g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO, "Signal: Stop capture");
+  capture_loop_stop();
+}
+
 #ifdef _WIN32
 #define TIME_GET() GetTickCount()
 #else
@@ -1701,6 +1790,9 @@ capture_loop_open_output(capture_options *capture_opts, int *save_file_fd,
 static gboolean
 capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct pcap_stat *stats)
 {
+#ifndef _WIN32
+  struct sigaction act;
+#endif
   time_t      upd_time, cur_time;
   time_t      start_time;
   int         err_close;
@@ -1723,48 +1815,66 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   *secondary_errmsg = '\0';
 
   /* init the loop data */
-  ld.go                 = TRUE;
-  ld.packet_count       = 0;
+  global_ld.go                 = TRUE;
+  global_ld.packet_count       = 0;
   if (capture_opts->has_autostop_packets)
-    ld.packet_max       = capture_opts->autostop_packets;
+    global_ld.packet_max       = capture_opts->autostop_packets;
   else
-    ld.packet_max       = 0;	/* no limit */
-  ld.err                = 0;	/* no error seen yet */
-  ld.wtap_linktype      = WTAP_ENCAP_UNKNOWN;
-  ld.pcap_err           = FALSE;
-  ld.from_cap_pipe      = FALSE;
-  ld.pdh                = NULL;
-  ld.cap_pipe_fd        = -1;
+    global_ld.packet_max       = 0;	/* no limit */
+  global_ld.err                = 0;	/* no error seen yet */
+  global_ld.wtap_linktype      = WTAP_ENCAP_UNKNOWN;
+  global_ld.pcap_err           = FALSE;
+  global_ld.from_cap_pipe      = FALSE;
+  global_ld.pdh                = NULL;
+  global_ld.cap_pipe_fd        = -1;
 #ifdef MUST_DO_SELECT
-  ld.pcap_fd            = 0;
+  global_ld.pcap_fd            = 0;
 #endif
 
   /* We haven't yet gotten the capture statistics. */
   *stats_known      = FALSE;
 
+#ifndef _WIN32
+  /*
+   * Catch SIGUSR1, so that we exit cleanly if the parent process
+   * kills us with it due to the user selecting "Capture->Stop".
+   */
+  act.sa_handler = capture_loop_stop_signal_handler;
+  /*
+   * Arrange that system calls not get restarted, because when
+   * our signal handler returns we don't want to restart
+   * a call that was waiting for packets to arrive.
+   */
+  act.sa_flags = 0;
+  sigemptyset(&act.sa_mask);
+  sigaction(SIGUSR1, &act, NULL);
+#endif /* _WIN32 */
+
   g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO, "Capture loop starting ...");
   capture_opts_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG, capture_opts);
 
   /* open the "input file" from network interface or capture pipe */
-  if (!capture_loop_open_input(capture_opts, &ld, errmsg, sizeof(errmsg),
+  if (!capture_loop_open_input(capture_opts, &global_ld, errmsg, sizeof(errmsg),
                                secondary_errmsg, sizeof(secondary_errmsg))) {
     goto error;
   }
 
   /* init the input filter from the network interface (capture pipe will do nothing) */
-  switch (capture_loop_init_filter(ld.pcap_h, ld.from_cap_pipe, capture_opts->iface, capture_opts->cfilter)) {
+  switch (capture_loop_init_filter(global_ld.pcap_h, global_ld.from_cap_pipe,
+                                   capture_opts->iface,
+				   capture_opts->cfilter)) {
 
   case INITFILTER_NO_ERROR:
     break;
 
   case INITFILTER_BAD_FILTER:
     cfilter_error = TRUE;
-    g_snprintf(errmsg, sizeof(errmsg), "%s", pcap_geterr(ld.pcap_h));
+    g_snprintf(errmsg, sizeof(errmsg), "%s", pcap_geterr(global_ld.pcap_h));
     goto error;
 
   case INITFILTER_OTHER_ERROR:
     g_snprintf(errmsg, sizeof(errmsg), "Can't install filter (%s).",
-               pcap_geterr(ld.pcap_h));
+               pcap_geterr(global_ld.pcap_h));
     g_snprintf(secondary_errmsg, sizeof(secondary_errmsg), "%s", please_report);
     goto error;
   }
@@ -1777,7 +1887,8 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
     }
 
     /* set up to write to the already-opened capture output file/files */
-    if (!capture_loop_init_output(capture_opts, save_file_fd, &ld, errmsg, sizeof(errmsg))) {
+    if (!capture_loop_init_output(capture_opts, save_file_fd, &global_ld,
+                                  errmsg, sizeof(errmsg))) {
       goto error;
     }
 
@@ -1794,7 +1905,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
        message to our parent so that they'll open the capture file and
        update its windows to indicate that we have a live capture in
        progress. */
-    libpcap_dump_flush(ld.pdh, NULL);
+    libpcap_dump_flush(global_ld.pdh, NULL);
     report_new_capture_file(capture_opts->save_file);
   }
 
@@ -1826,14 +1937,15 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
 
   /* WOW, everything is prepared! */
   /* please fasten your seat belts, we will enter now the actual capture loop */
-  while (ld.go) {
+  while (global_ld.go) {
     /* dispatch incoming packets */
-    inpkts = capture_loop_dispatch(capture_opts, &ld, errmsg, sizeof(errmsg));
+    inpkts = capture_loop_dispatch(capture_opts, &global_ld, errmsg,
+                                   sizeof(errmsg));
 
 #ifdef _WIN32
     /* any news from our parent (signal pipe)? -> just stop the capture */
     if (!signal_pipe_check_running()) {
-      ld.go = FALSE;
+      global_ld.go = FALSE;
     }
 #endif
 
@@ -1842,41 +1954,41 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
 
       /* check capture size condition */
       if (cnd_autostop_size != NULL &&
-          cnd_eval(cnd_autostop_size, (guint32)ld.bytes_written)){
+          cnd_eval(cnd_autostop_size, (guint32)global_ld.bytes_written)){
         /* Capture size limit reached, do we have another file? */
         if (capture_opts->multi_files_on) {
           if (cnd_autostop_files != NULL &&
               cnd_eval(cnd_autostop_files, ++autostop_files)) {
              /* no files left: stop here */
-            ld.go = FALSE;
+            global_ld.go = FALSE;
             continue;
           }
 
           /* Switch to the next ringbuffer file */
-          if (ringbuf_switch_file(&ld.pdh, &capture_opts->save_file,
-              &save_file_fd, &ld.bytes_written, &ld.err)) {
+          if (ringbuf_switch_file(&global_ld.pdh, &capture_opts->save_file,
+              &save_file_fd, &global_ld.bytes_written, &global_ld.err)) {
             /* File switch succeeded: reset the conditions */
             cnd_reset(cnd_autostop_size);
             if (cnd_file_duration) {
               cnd_reset(cnd_file_duration);
             }
-            libpcap_dump_flush(ld.pdh, NULL);
+            libpcap_dump_flush(global_ld.pdh, NULL);
             report_packet_count(inpkts_to_sync_pipe);
             inpkts_to_sync_pipe = 0;
             report_new_capture_file(capture_opts->save_file);
           } else {
             /* File switch failed: stop here */
-            ld.go = FALSE;
+            global_ld.go = FALSE;
             continue;
           }
         } else {
           /* single file, stop now */
-          ld.go = FALSE;
+          global_ld.go = FALSE;
           continue;
         }
       } /* cnd_autostop_size */
       if (capture_opts->output_to_pipe) {
-        libpcap_dump_flush(ld.pdh, NULL);
+        libpcap_dump_flush(global_ld.pdh, NULL);
       }
     } /* inpkts */
 
@@ -1898,7 +2010,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
       /* Let the parent process know. */
       if (inpkts_to_sync_pipe) {
         /* do sync here */
-        libpcap_dump_flush(ld.pdh, NULL);
+        libpcap_dump_flush(global_ld.pdh, NULL);
 
         /* Send our parent a message saying we've written out "inpkts_to_sync_pipe"
            packets to the capture file. */
@@ -1910,7 +2022,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
       /* check capture duration condition */
       if (cnd_autostop_duration != NULL && cnd_eval(cnd_autostop_duration)) {
         /* The maximum capture time has elapsed; stop the capture. */
-        ld.go = FALSE;
+        global_ld.go = FALSE;
         continue;
       }
 
@@ -1921,35 +2033,36 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
           if (cnd_autostop_files != NULL &&
               cnd_eval(cnd_autostop_files, ++autostop_files)) {
             /* no files left: stop here */
-            ld.go = FALSE;
+            global_ld.go = FALSE;
             continue;
           }
 
           /* Switch to the next ringbuffer file */
-          if (ringbuf_switch_file(&ld.pdh, &capture_opts->save_file,
-                                  &save_file_fd, &ld.bytes_written, &ld.err)) {
+          if (ringbuf_switch_file(&global_ld.pdh, &capture_opts->save_file,
+                                  &save_file_fd, &global_ld.bytes_written,
+				  &global_ld.err)) {
             /* file switch succeeded: reset the conditions */
             cnd_reset(cnd_file_duration);
             if(cnd_autostop_size)
               cnd_reset(cnd_autostop_size);
-            libpcap_dump_flush(ld.pdh, NULL);
+            libpcap_dump_flush(global_ld.pdh, NULL);
             report_packet_count(inpkts_to_sync_pipe);
             inpkts_to_sync_pipe = 0;
             report_new_capture_file(capture_opts->save_file);
           } else {
             /* File switch failed: stop here */
-            ld.go = FALSE;
+            global_ld.go = FALSE;
             continue;
           }
         } else {
           /* single file, stop now */
-          ld.go = FALSE;
+          global_ld.go = FALSE;
           continue;
         }
       } /* cnd_file_duration */
     }
 
-  } /* while (ld.go) */
+  } /* while (global_ld.go) */
 
   g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO, "Capture loop stopping ...");
 
@@ -1964,7 +2077,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
     cnd_delete(cnd_autostop_duration);
 
   /* did we had a pcap (input) error? */
-  if (ld.pcap_err) {
+  if (global_ld.pcap_err) {
     /* On Linux, if an interface goes down while you're capturing on it,
        you'll get a "recvfrom: Network is down" error (ENETDOWN).
        (At least you will if strerror() doesn't show a local translation
@@ -1979,7 +2092,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
        These should *not* be reported to the Wireshark developers. */
     char *cap_err_str;
 
-    cap_err_str = pcap_geterr(ld.pcap_h);
+    cap_err_str = pcap_geterr(global_ld.pcap_h);
     if (strcmp(cap_err_str, "recvfrom: Network is down") == 0 ||
         strcmp(cap_err_str, "read: Device not configured") == 0 ||
         strcmp(cap_err_str, "read: I/O error") == 0) {
@@ -1992,22 +2105,22 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
       report_capture_error(errmsg, please_report);
     }
   }
-  else if (ld.from_cap_pipe && ld.cap_pipe_err == PIPERR)
+  else if (global_ld.from_cap_pipe && global_ld.cap_pipe_err == PIPERR)
     report_capture_error(errmsg, "");
 
   /* did we had an error while capturing? */
-  if (ld.err == 0) {
+  if (global_ld.err == 0) {
     write_ok = TRUE;
   } else {
-    capture_loop_get_errmsg(errmsg, sizeof(errmsg), capture_opts->save_file, ld.err,
-			      FALSE);
+    capture_loop_get_errmsg(errmsg, sizeof(errmsg), capture_opts->save_file,
+                            global_ld.err, FALSE);
     report_capture_error(errmsg, please_report);
     write_ok = FALSE;
   }
 
   if (capture_opts->saving_to_file) {
     /* close the wiretap (output) file */
-    close_ok = capture_loop_close_output(capture_opts, &ld, &err_close);
+    close_ok = capture_loop_close_output(capture_opts, &global_ld, &err_close);
   } else
     close_ok = TRUE;
 
@@ -2036,24 +2149,24 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
    */
 
   /* get packet drop statistics from pcap */
-  if(ld.pcap_h != NULL) {
-    g_assert(!ld.from_cap_pipe);
+  if(global_ld.pcap_h != NULL) {
+    g_assert(!global_ld.from_cap_pipe);
     /* Get the capture statistics, so we know how many packets were
        dropped. */
-    if (pcap_stats(ld.pcap_h, stats) >= 0) {
+    if (pcap_stats(global_ld.pcap_h, stats) >= 0) {
       *stats_known = TRUE;
       /* Let the parent process know. */
       report_packet_drops(stats->ps_drop);
     } else {
       g_snprintf(errmsg, sizeof(errmsg),
 		"Can't get packet-drop statistics: %s",
-		pcap_geterr(ld.pcap_h));
+		pcap_geterr(global_ld.pcap_h));
       report_capture_error(errmsg, please_report);
     }
   }
 
   /* close the input file (pcap or capture pipe) */
-  capture_loop_close_input(&ld);
+  capture_loop_close_input(&global_ld);
 
   g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO, "Capture loop stopped!");
 
@@ -2068,13 +2181,13 @@ error:
     /* We can't use the save file, and we have no FILE * for the stream
        to close in order to close it, so close the FD directly. */
     if(save_file_fd != -1) {
-      eth_close(save_file_fd);
+      ws_close(save_file_fd);
     }
 
     /* We couldn't even start the capture, so get rid of the capture
        file. */
     if(capture_opts->save_file != NULL) {
-      eth_unlink(capture_opts->save_file);
+      ws_unlink(capture_opts->save_file);
       g_free(capture_opts->save_file);
     }
   }
@@ -2085,7 +2198,7 @@ error:
     report_capture_error(errmsg, secondary_errmsg);
 
   /* close the input file (pcap or cap_pipe) */
-  capture_loop_close_input(&ld);
+  capture_loop_close_input(&global_ld);
 
   g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO, "Capture loop stopped with error");
 
@@ -2096,10 +2209,10 @@ error:
 static void capture_loop_stop(void)
 {
 #ifdef HAVE_PCAP_BREAKLOOP
-  if(ld.pcap_h != NULL)
-    pcap_breakloop(ld.pcap_h);
+  if(global_ld.pcap_h != NULL)
+    pcap_breakloop(global_ld.pcap_h);
 #endif
-  ld.go = FALSE;
+  global_ld.go = FALSE;
 }
 
 
@@ -2170,6 +2283,13 @@ capture_loop_packet_cb(u_char *user, const struct pcap_pkthdr *phdr,
   loop_data *ld = (void *) user;
   int err;
 
+  /* if the user told us to stop after x packets, do we already have enough? */
+  if ((ld->packet_max > 0) && (ld->packet_count >= ld->packet_max))
+  {
+     ld->go = FALSE;
+     return;
+  }
+
   /* We may be called multiple times from pcap_dispatch(); if we've set
      the "stop capturing" flag, ignore this packet, as we're not
      supposed to be saving any more packets. */
@@ -2183,14 +2303,8 @@ capture_loop_packet_cb(u_char *user, const struct pcap_pkthdr *phdr,
     if (!libpcap_write_packet(ld->pdh, phdr, pd, &ld->bytes_written, &err)) {
       ld->go = FALSE;
       ld->err = err;
-    } else {
+    } else
       ld->packet_count++;
-      /* if the user told us to stop after x packets, do we already have enough? */
-      if ((ld->packet_max > 0) && (ld->packet_count >= ld->packet_max))
-      {
-        ld->go = FALSE;
-      }
-    }
   }
 }
 
@@ -2237,7 +2351,7 @@ main(int argc, char *argv[])
     OPTSTRING_INIT OPTSTRING_WIN32;
 
 #ifdef DEBUG_CHILD_DUMPCAP
-  if ((debug_log = fopen("dumpcap_debug_log.tmp","w")) == NULL) {
+  if ((debug_log = ws_fopen("dumpcap_debug_log.tmp","w")) == NULL) {
           fprintf (stderr, "Unable to open debug log file !\n");
           exit (1);
   }
@@ -2309,21 +2423,15 @@ main(int argc, char *argv[])
   WSAStartup( MAKEWORD( 1, 1 ), &wsaData );
 
   /* Set handler for Ctrl+C key */
-  SetConsoleCtrlHandler(capture_cleanup_handler, TRUE);
+  SetConsoleCtrlHandler(capture_cleanup, TRUE);
 #else
   /* Catch SIGINT and SIGTERM and, if we get either of them, clean up
      and exit. */
-  action.sa_handler = capture_cleanup_handler;
-  /*
-   * Arrange that system calls not get restarted, because when
-   * our signal handler returns we don't want to restart
-   * a call that was waiting for packets to arrive.
-   */
+  action.sa_handler = capture_cleanup;
   action.sa_flags = 0;
   sigemptyset(&action.sa_mask);
   sigaction(SIGTERM, &action, NULL);
   sigaction(SIGINT, &action, NULL);
-  sigaction(SIGPIPE, &action, NULL);
   sigaction(SIGHUP, NULL, &oldaction);
   if (oldaction.sa_handler == SIG_DFL)
     sigaction(SIGHUP, &action, NULL);
@@ -2385,7 +2493,7 @@ main(int argc, char *argv[])
   /*                                                                   */
   /*        It is therefore conceivable that if dumpcap somehow hangs  */
   /*        in pcap_open_live or before that wireshark will not        */
-  /*        be able to stop dumpcap using a signal (INT, TERM, etc).  */
+  /*        be able to stop dumpcap using a signal (USR1, TERM, etc).  */
   /*        In this case, exiting wireshark will kill the child        */
   /*        dumpcap process.                                           */
   /*                                                                   */
@@ -2411,17 +2519,17 @@ main(int argc, char *argv[])
   relinquish_privs_except_capture();
 #endif
 
-  /* Set the initial values in the capture_opts. This might be overwritten
+  /* Set the initial values in the capture options. This might be overwritten
      by the command line parameters. */
-  capture_opts_init(capture_opts, NULL);
+  capture_opts_init(&global_capture_opts, NULL);
 
   /* Default to capturing the entire packet. */
-  capture_opts->snaplen             = WTAP_MAX_PACKET_SIZE;
+  global_capture_opts.snaplen             = WTAP_MAX_PACKET_SIZE;
 
   /* We always save to a file - if no file was specified, we save to a
      temporary file. */
-  capture_opts->saving_to_file      = TRUE;
-  capture_opts->has_ring_num_files  = TRUE;
+  global_capture_opts.saving_to_file      = TRUE;
+  global_capture_opts.has_ring_num_files  = TRUE;
 
   /* Now get our args */
   while ((opt = getopt(argc, argv, optstring)) != -1) {
@@ -2468,7 +2576,7 @@ main(int argc, char *argv[])
 #ifdef _WIN32
       case 'B':        /* Buffer size */
 #endif /* _WIN32 */
-        status = capture_opts_add_opt(capture_opts, opt, optarg, &start_capture);
+        status = capture_opts_add_opt(&global_capture_opts, opt, optarg, &start_capture);
         if(status != 0) {
           exit_main(status);
         }
@@ -2551,56 +2659,56 @@ main(int argc, char *argv[])
     /* We're supposed to list the link-layer types for an interface;
        did the user also specify a capture file to be read? */
     /* No - did they specify a ring buffer option? */
-    if (capture_opts->multi_files_on) {
+    if (global_capture_opts.multi_files_on) {
       cmdarg_err("Ring buffer requested, but a capture isn't being done.");
       exit_main(1);
     }
   } else {
     /* No - was the ring buffer option specified and, if so, does it make
        sense? */
-    if (capture_opts->multi_files_on) {
+    if (global_capture_opts.multi_files_on) {
       /* Ring buffer works only under certain conditions:
 	 a) ring buffer does not work with temporary files;
 	 b) it makes no sense to enable the ring buffer if the maximum
 	    file size is set to "infinite". */
-      if (capture_opts->save_file == NULL) {
+      if (global_capture_opts.save_file == NULL) {
 	cmdarg_err("Ring buffer requested, but capture isn't being saved to a permanent file.");
-	capture_opts->multi_files_on = FALSE;
+	global_capture_opts.multi_files_on = FALSE;
       }
-      if (!capture_opts->has_autostop_filesize && !capture_opts->has_file_duration) {
+      if (!global_capture_opts.has_autostop_filesize && !global_capture_opts.has_file_duration) {
 	cmdarg_err("Ring buffer requested, but no maximum capture file size or duration were specified.");
 /* XXX - this must be redesigned as the conditions changed */
-/*	capture_opts->multi_files_on = FALSE;*/
+/*	global_capture_opts.multi_files_on = FALSE;*/
       }
     }
   }
 
-  if (capture_opts_trim_iface(capture_opts, NULL) == FALSE) {
+  if (capture_opts_trim_iface(&global_capture_opts, NULL) == FALSE) {
     /* cmdarg_err() already called .... */
     exit_main(1);
   }
 
   /* Let the user know what interface was chosen. */
   /* get_interface_descriptive_name() is not available! */
-  g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG, "Interface: %s\n", capture_opts->iface);
+  g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG, "Interface: %s\n", global_capture_opts.iface);
 
   if (list_interfaces) {
     status = capture_opts_list_interfaces(machine_readable);
     exit_main(status);
   } else if (list_link_layer_types) {
-    status = capture_opts_list_link_layer_types(capture_opts, machine_readable);
+    status = capture_opts_list_link_layer_types(&global_capture_opts, machine_readable);
     exit_main(status);
   } else if (print_statistics) {
-    status = capture_opts_print_statistics(machine_readable);
+    status = print_statistics_loop(machine_readable);
     exit_main(status);
   }
 
-  capture_opts_trim_snaplen(capture_opts, MIN_PACKET_SIZE);
-  capture_opts_trim_ring_num_files(capture_opts);
+  capture_opts_trim_snaplen(&global_capture_opts, MIN_PACKET_SIZE);
+  capture_opts_trim_ring_num_files(&global_capture_opts);
 
   /* Now start the capture. */
 
-  if(capture_loop_start(capture_opts, &stats_known, &stats) == TRUE) {
+  if(capture_loop_start(&global_capture_opts, &stats_known, &stats) == TRUE) {
     /* capture ok */
     exit_main(0);
   } else {
@@ -2761,11 +2869,11 @@ report_capture_error(const char *error_msg, const char *secondary_error_msg)
 }
 
 void
-report_packet_drops(int drops)
+report_packet_drops(guint32 drops)
 {
     char tmp[SP_DECISIZE+1+1];
 
-    g_snprintf(tmp, sizeof(tmp), "%d", drops);
+    g_snprintf(tmp, sizeof(tmp), "%u", drops);
 
     if(capture_child) {
         g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG, "Packets dropped: %s", tmp);
