@@ -1,7 +1,7 @@
 /* packet-3com-njack.c
  * Routines for the disassembly of the 3com NetworkJack management protocol
  *
- * $Id: packet-3com-njack.c 20277 2007-01-02 22:49:57Z gerald $
+ * $Id: packet-3com-njack.c 26121 2008-09-02 19:08:29Z wmeier $
  *
  * Copyright 2005 Joerg Mayer (see AUTHORS file)
  *
@@ -59,6 +59,7 @@ Specs:
 #include <epan/packet.h>
 #include <epan/emem.h>
 
+
 /* protocol handles */
 static int proto_njack = -1;
 
@@ -228,6 +229,7 @@ static const value_string njack_dhcpcontrol[] = {
 /* End General settings TAB */
 
 /* Port settings TAB */
+#if 0
 static const true_false_string tfs_port_state = {
 	"Disable",
 	"Enable"
@@ -248,6 +250,7 @@ static const true_false_string tfs_port_duplex = {
 	"duplex"
 };
 
+#endif
 /* End Port settings TAB */
 
 /* Hardware Settings TAB */
@@ -304,6 +307,7 @@ static const value_string njack_snmpwrite[] = {
 	{ 0,	NULL }
 };
 
+#if 0
 static const value_string njack_snmptrap[] = {
 	{ 0,	"Disable" },
 	{ 1,	"Enable" },
@@ -338,6 +342,7 @@ static const value_string njack_authfailtrap[] = {
 
 	{ 0,	NULL }
 };
+#endif
 /* End SNMP TAB */
 
 static int
@@ -546,7 +551,7 @@ verify_password(tvbuff_t *tvb, const char *password)
 	md5_append(&md_ctx, digest, 16);
 	md5_append(&md_ctx, packetdata + 28, length - 28);
 	md5_finish(&md_ctx, digest);
-	fprintf(stderr, "Calclulated digest: "); /* debugging */
+	fprintf(stderr, "Calculated digest: "); /* debugging */
 	for (i = 0; i < 16; i++) {
 		fprintf(stderr, "%02X", digest[i]); /* debugging */
 		if (digest[i] != *(packetdata + 12 + i)) {
@@ -574,7 +579,7 @@ dissect_njack(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
 		col_set_str(pinfo->cinfo, COL_PROTOCOL, PROTO_SHORT_NAME);
 	if (check_col(pinfo->cinfo, COL_INFO))
-		col_add_fstr(pinfo->cinfo, COL_INFO, val_to_str(packet_type,
+		col_add_str(pinfo->cinfo, COL_INFO, val_to_str(packet_type,
 			njack_type_vals, "Type 0x%02x"));
 
 	if (tree) {
@@ -645,8 +650,8 @@ static gboolean
 test_njack(tvbuff_t *tvb)
 {
 	/* We need at least 'NJ200' + 1 Byte packet type */
-	if ( !tvb_bytes_exist(tvb, 0, 6) ||
-		    g_strncasecmp((const char *)tvb_get_ptr(tvb, 0, 5), "NJ200", 5) ) {
+	if ( tvb_length(tvb) < 6 ||
+		    g_ascii_strncasecmp((const char *)tvb_get_ptr(tvb, 0, 5), "NJ200", 5) ) {
         	return FALSE;
 	}
 	return TRUE;
@@ -671,8 +676,6 @@ dissect_njack_static(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	return dissect_njack(tvb, pinfo, tree);
 }
 
-void proto_register_njack(void);
-
 void
 proto_register_njack(void)
 {
@@ -781,14 +784,11 @@ proto_register_njack(void)
 		&ett_njack_tlv_header,
 	};
 
-        proto_njack = proto_register_protocol(PROTO_LONG_NAME,
-	    PROTO_SHORT_NAME, "njack");
+        proto_njack = proto_register_protocol(PROTO_LONG_NAME, PROTO_SHORT_NAME, "njack");
         proto_register_field_array(proto_njack, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 }
 
-void proto_reg_handoff_njack(void);
-
 void
 proto_reg_handoff_njack(void)
 {
