@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2005 by Alejandro Vaquero <alejandro.vaquero@verso.com>
  *
- * $Id: packet-actrace.c 23192 2007-10-15 20:11:25Z etxrab $
+ * $Id: packet-actrace.c 26124 2008-09-02 21:40:46Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -434,12 +434,6 @@ static actrace_info_t *actrace_pi;
  */
 static guint global_actrace_udp_port = UDP_PORT_ACTRACE;
 
-/*
- * Variables to allow for proper deletion of dissector registration when
- * the user changes port from the gui.
- */
-static guint actrace_udp_port = 0;
-
 /* Some basic utility functions that are specific to this dissector */
 static int is_actrace(tvbuff_t *tvb, gint offset);
 
@@ -453,7 +447,7 @@ static void dissect_actrace_isdn(tvbuff_t *tvb, packet_info *pinfo, proto_tree *
 /************************************************************************
  * dissect_actrace - The dissector for the AudioCodes Trace prtocol
  ************************************************************************/
-static void dissect_actrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+static int dissect_actrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	proto_tree *actrace_tree;
 	proto_item *ti;
@@ -462,15 +456,6 @@ static void dissect_actrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	/* Initialize variables */
 	actrace_tree = NULL;
 
-	/*
-	 * Set the columns now, so that they'll be set correctly if we throw
-	 * an exception.  We can set them later as well....
-	 */
-	if (check_col(pinfo->cinfo, COL_PROTOCOL))
-		col_set_str(pinfo->cinfo, COL_PROTOCOL, "AC_TRACE");
-	if (check_col(pinfo->cinfo, COL_INFO))
-		col_clear(pinfo->cinfo, COL_INFO);
-
 	/*
 	 * Check to see whether we're really dealing with AC trace by looking
 	 * for a valid "source" and fixed len for CAS; and the direction for ISDN.
@@ -479,6 +464,15 @@ static void dissect_actrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	actrace_protocol = is_actrace(tvb, 0);
 	if (actrace_protocol != NOT_ACTRACE)
 	{
+		/*
+		 * Set the columns now, so that they'll be set correctly if we throw
+		 * an exception.  We can set them later as well....
+		 */
+		if (check_col(pinfo->cinfo, COL_PROTOCOL))
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "AC_TRACE");
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_clear(pinfo->cinfo, COL_INFO);
+
 		if (tree)
 		{
 			/* Create our actrace subtree */
@@ -495,10 +489,10 @@ static void dissect_actrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				dissect_actrace_isdn(tvb, pinfo, tree, actrace_tree);
 				break;
 		}
-	} else {
-		if (check_col(pinfo->cinfo, COL_INFO))
-			col_set_str(pinfo->cinfo, COL_INFO, "Non CAS or ISDN AudioCodes trace message");
+		return tvb_length(tvb);
 	}
+
+	return 0;
 }
 
 /* Dissect an individual actrace CAS message */
@@ -807,15 +801,15 @@ void proto_register_actrace(void)
 /* The registration hand-off routine */
 void proto_reg_handoff_actrace(void)
 {
-	static int actrace_prefs_initialized = FALSE;
+	static gboolean actrace_prefs_initialized = FALSE;
 	static dissector_handle_t actrace_handle;
-
-	/* Get a handle for the lapd dissector. */
-	lapd_handle = find_dissector("lapd");
+	static guint actrace_udp_port = 0;
 
 	if (!actrace_prefs_initialized)
 	{
-		actrace_handle = create_dissector_handle(dissect_actrace, proto_actrace);
+		actrace_handle = new_create_dissector_handle(dissect_actrace, proto_actrace);
+		/* Get a handle for the lapd dissector. */
+		lapd_handle = find_dissector("lapd");
 		actrace_prefs_initialized = TRUE;
 	}
 	else
