@@ -3,7 +3,7 @@
  * Copyright 2004, Jelmer Vernooij <jelmer@samba.org>
  * Copyright 2000, Ralf Hoelzer <ralf@well.com>
  *
- * $Id: packet-aim-ssi.c 22742 2007-08-30 05:22:25Z etxrab $
+ * $Id: packet-aim-ssi.c 25822 2008-07-25 12:44:55Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -75,12 +75,16 @@ static int hf_aim_fnac_subtype_ssi_version = -1;
 static int hf_aim_fnac_subtype_ssi_numitems = -1;
 static int hf_aim_fnac_subtype_ssi_last_change_time = -1;
 static int hf_aim_fnac_subtype_ssi_buddyname_len = -1;
+static int hf_aim_fnac_subtype_ssi_buddyname_len8 = -1;
 static int hf_aim_fnac_subtype_ssi_buddyname = -1;
 static int hf_aim_fnac_subtype_ssi_gid = -1;
 static int hf_aim_fnac_subtype_ssi_bid = -1;
 static int hf_aim_fnac_subtype_ssi_type = -1;
 static int hf_aim_fnac_subtype_ssi_tlvlen = -1;
 static int hf_aim_fnac_subtype_ssi_data = -1;
+static int hf_aim_fnac_subtype_ssi_reason_str_len = -1;
+static int hf_aim_fnac_subtype_ssi_reason_str = -1;
+static int hf_aim_fnac_subtype_ssi_grant_ft_auth_unkn = -1;
 
 /* Initialize the subtree pointers */
 static gint ett_aim_ssi      = -1;
@@ -154,14 +158,33 @@ static int dissect_aim_ssi_was_added(tvbuff_t *tvb, packet_info *pinfo, proto_tr
 	return dissect_aim_buddyname(tvb, pinfo, 0, ssi_tree);
 }
 
-static int dissect_aim_snac_ssi_list(tvbuff_t *tvb, packet_info *pinfo _U_, 
-									 proto_tree *tree)
 
+static int dissect_aim_snac_ssi_time_and_items_num(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
+{
+	int offset = 0;
+
+	/* get timestamp */
+	nstime_t tmptime;
+	tmptime.secs = tvb_get_ntohl(tvb, offset);
+	tmptime.nsecs = 0;
+	proto_tree_add_time(tree, hf_aim_fnac_subtype_ssi_last_change_time, tvb, offset, 4, &tmptime);
+	offset += 4;
+
+	/* get number of SSI items */
+	proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_numitems, tvb, offset, 2,     FALSE);
+	offset += 2;
+
+	return offset;
+}
+
+
+static int dissect_aim_snac_ssi_list(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 {
 	int offset = 0;
 	proto_item *ti;
 	proto_tree *ssi_entry = NULL;
 	guint16 num_items, i;
+	nstime_t tmptime;
 
 	/* SSI Version */
 	proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_version, tvb, offset, 1,
@@ -175,11 +198,59 @@ static int dissect_aim_snac_ssi_list(tvbuff_t *tvb, packet_info *pinfo _U_,
 	offset += 2;
 
 	for(i = 0; i < num_items; i++) {
-		ti = proto_tree_add_text(tree, tvb, offset, tvb_get_ntohs(tvb, offset+10)+10, "SSI Entry");
+		/* size of SSI entry can be calculated as:
+		 *   sizeof(buddy name length field) = sizeof(guint16) = 2
+		 * + sizeof(buddy name string) = buddy name length field = N
+		 * + sizeof(group ID) = sizeof(guint16) = 2
+		 * + sizeof(buddy ID) = sizeof(guint16) = 2
+		 * + sizeof(buddy type) = sizeof(guint16) = 2
+		 * + sizeof(TLV length) = sizeof(guint16) = 2
+		 * + sizeof(TLVs) = TLV length = M
+		 * = 2 + N + 2 * 4 + M
+		 */
+		gint ssi_entry_size = 2 + tvb_get_ntohs(tvb, offset) + 2 * 3;
+		ssi_entry_size += tvb_get_ntohs(tvb, offset + ssi_entry_size) + 2;
+		ti = proto_tree_add_text(tree, tvb, offset, ssi_entry_size, "SSI Entry");
 		ssi_entry = proto_item_add_subtree(ti, ett_aim_ssi);
 		offset = dissect_ssi_item(tvb, pinfo, offset, ssi_entry);
 	}
-	proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_last_change_time, tvb, offset, 4, FALSE);
+	tmptime.secs = tvb_get_ntohl(tvb, offset);
+	tmptime.nsecs = 0;
+	proto_tree_add_time(tree, hf_aim_fnac_subtype_ssi_last_change_time, tvb, offset, 4, &tmptime);
+
+	return offset;
+}
+
+static int dissect_aim_snac_ssi_future_auth(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
+{
+	int offset = 0;
+	guint16 reason_length;
+	guint16 unknown;
+
+	/* get buddy length (1 byte) */
+	guint8 buddyname_length = tvb_get_guint8(tvb, offset);
+	proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_buddyname_len8, tvb, offset, 1, FALSE);
+	offset += 1;
+	/* show buddy name */
+	if (buddyname_length > 0) {
+		proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_buddyname, tvb, offset, buddyname_length, FALSE);
+		offset += buddyname_length;
+	}
+
+	/* get reason message length (2 bytes) */
+	reason_length = tvb_get_ntohs(tvb, offset);
+	proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_reason_str_len, tvb, offset, 2, FALSE);
+	offset += 2;
+	/* show reason message if present */
+	if (reason_length > 0) {
+		proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_reason_str, tvb, offset, reason_length, FALSE);
+		offset += reason_length;
+	}
+	/* unknown (always 0x0000 ???) */
+	unknown = tvb_get_ntohs(tvb, offset);
+	proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_grant_ft_auth_unkn, tvb, offset, 2, FALSE);
+	offset += 2;
+
 	return offset;
 }
 
@@ -188,18 +259,18 @@ static const aim_subtype aim_fnac_family_ssi[] = {
 	{ 0x0002, "Request Rights", NULL },
 	{ 0x0003, "Rights Info", dissect_aim_ssi_rightsinfo },
 	{ 0x0004, "Request List (first time)", NULL },
-	{ 0x0005, "Request List", NULL },
+	{ 0x0005, "Request List", dissect_aim_snac_ssi_time_and_items_num },
 	{ 0x0006, "List", dissect_aim_snac_ssi_list },
 	{ 0x0007, "Activate", NULL },
 	{ 0x0008, "Add Buddy", dissect_ssi_ssi_item },
 	{ 0x0009, "Modify Buddy", dissect_ssi_ssi_item },
 	{ 0x000a, "Delete Buddy", dissect_ssi_ssi_item },
-	{ 0x000e, "Server Ack", NULL },
-	{ 0x000f, "No List", NULL },
+	{ 0x000e, "Server Ack", dissect_aim_ssi_result },
+	{ 0x000f, "No List", dissect_aim_snac_ssi_time_and_items_num },
 	{ 0x0011, "Edit Start", NULL },
 	{ 0x0012, "Edit Stop", NULL },
-	{ 0x0014, "Grant Future Authorization to Buddy", NULL },
-	{ 0x0015, "Future Authorization Granted", NULL },
+	{ 0x0014, "Grant Future Authorization to Buddy", dissect_aim_snac_ssi_future_auth },
+	{ 0x0015, "Future Authorization Granted", dissect_aim_snac_ssi_future_auth },
 	{ 0x0018, "Send Authentication Request", NULL },
 	{ 0x0019, "Authentication Request", NULL },
 	{ 0x001a, "Send Authentication Reply", NULL },
@@ -223,7 +294,7 @@ proto_register_aim_ssi(void)
 			{ "SSI Object count", "aim_ssi.fnac.numitems", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
 		},
 		{ &hf_aim_fnac_subtype_ssi_last_change_time,
-			{ "SSI Last Change Time", "aim_ssi.fnac.last_change_time", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }
+			{ "SSI Last Change Time", "aim_ssi.fnac.last_change_time", FT_ABSOLUTE_TIME, BASE_NONE, NULL, 0x0, "", HFILL }
 		},
 		{ &hf_aim_fnac_subtype_ssi_buddyname_len,
 			{ "SSI Buddy Name length", "aim_ssi.fnac.buddyname_len", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
@@ -246,6 +317,18 @@ proto_register_aim_ssi(void)
 		{ &hf_aim_fnac_subtype_ssi_data,
 			{ "SSI Buddy Data", "aim_ssi.fnac.data", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
 		},
+		{ &hf_aim_fnac_subtype_ssi_buddyname_len8,
+			{ "SSI Buddy Name length", "aim_ssi.fnac.buddyname_len8", FT_UINT8, BASE_HEX, NULL, 0x0, "", HFILL }
+		},
+		{ &hf_aim_fnac_subtype_ssi_reason_str_len,
+			{ "Reason Message length", "aim_ssi.fnac.reason_len", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+		},
+		{ &hf_aim_fnac_subtype_ssi_reason_str,
+			{ "Reason Message", "aim_ssi.fnac.reason", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }
+		},
+		{ &hf_aim_fnac_subtype_ssi_grant_ft_auth_unkn,
+			{ "Unknown", "aim_ssi.fnac.grant_ft_auth_unkn", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+		},
 	};
 
 	/* Setup protocol subtree array */
