@@ -4,7 +4,7 @@
  * Copyright 2003, Michael Lum <mlum [AT] telostech.com>
  * In association with Telos Technology Inc.
  *
- * $Id: packet-ansi_683.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-ansi_683.c 25274 2008-05-11 17:27:41Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1499,7 +1499,7 @@ msg_protocap_rsp(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset)
 
     proto_tree_add_none_format(tree, hf_ansi_683_none,
 	tvb, offset, 1,
-	"Mobile station manufacturers model number (%d)",
+	"Mobile station manufacturer's model number (%d)",
 	oct);
 
     offset++;
