@@ -1,7 +1,7 @@
 /* packet-arp.c
  * Routines for ARP packet disassembly (RFC 826)
  *
- * $Id: packet-arp.c 23983 2008-01-02 10:35:20Z martinm $
+ * $Id: packet-arp.c 25395 2008-05-30 00:01:24Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -421,7 +421,7 @@ static gint address_equal_func(gconstpointer v, gconstpointer v2)
    the same IP address - report if found (+ return TRUE and set out param) */
 static gboolean check_for_duplicate_addresses(packet_info *pinfo, proto_tree *tree,
                                               tvbuff_t *tvb,
-                                              const gchar *mac, guint32 ip,
+                                              const guint8 *mac, guint32 ip,
                                               guint32 *duplicate_ip)
 {
     struct   address_hash_value *value;
