@@ -1,7 +1,7 @@
 /* packet-beep.c
  * Routines for BEEP packet disassembly
  *
- * $Id: packet-beep.c 21210 2007-03-26 11:06:26Z sahlberg $
+ * $Id: packet-beep.c 26121 2008-09-02 19:08:29Z wmeier $
  *
  * Copyright (c) 2000 by Richard Sharpe <rsharpe@ns.aus.com>
  * Modified 2001 Darren New <dnew@invisible.net> for BEEP.
@@ -47,7 +47,6 @@
 #define TCP_PORT_BEEP 10288
 void proto_reg_handoff_beep(void);
 
-
 static guint global_beep_tcp_port = TCP_PORT_BEEP;
 static int global_beep_strict_term = TRUE;
 
@@ -213,50 +212,56 @@ static int
 dissect_beep_more(tvbuff_t *tvb, int offset,
 		  proto_tree *tree)
 {
-
+  proto_item *hidden_item;
+  int ret = 0;
 
   switch (beep_get_more(tvb_get_guint8(tvb, offset))) {
 
   case BEEP_COMPLETE:
 
     if (tree) {
-      proto_tree_add_boolean_hidden(tree, hf_beep_complete, tvb, offset, 1, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_complete, tvb, offset, 1, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
       proto_tree_add_text(tree, tvb, offset, 1, "More: Complete");
     }
 
-    return 0;
+    ret = 0;
 
     break;
 
   case BEEP_INTERMEDIATE:
 
     if (tree) {
-      proto_tree_add_boolean_hidden(tree, hf_beep_intermediate, tvb, offset, 1, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_intermediate, tvb, offset, 1, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
       proto_tree_add_text(tree, tvb, offset, 1, "More: Intermediate");
     }
 
-    return 1;
+    ret = 1;
 
     break;
 
   default:
 
     if (tree) {
-      proto_tree_add_boolean_hidden(tree, hf_beep_proto_viol, tvb, offset, 1, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_proto_viol, tvb, offset, 1, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
       proto_tree_add_text(tree, tvb, offset, 1, "PROTOCOL VIOLATION: Expected More Flag (* or .)");
     }
 
-    return -1;
+    ret = -1;
 
     break;
   }
 
+  return ret;
 }
 
 #if 0
 static void dissect_beep_status(tvbuff_t *tvb, int offset,
 				proto_tree *tree)
 {
+	proto_item *hidden_item;
 
   /* FIXME: We should return a value to indicate all OK. */
 
@@ -265,7 +270,8 @@ static void dissect_beep_status(tvbuff_t *tvb, int offset,
   case '+':
 
     if (tree) {
-      proto_tree_add_boolean_hidden(tree, hf_beep_positive, tvb, offset, 1, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_positive, tvb, offset, 1, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
       proto_tree_add_text(tree, tvb, offset, 1, "Status: Positive");
     }
 
@@ -274,7 +280,8 @@ static void dissect_beep_status(tvbuff_t *tvb, int offset,
   case '-':
 
     if (tree) {
-      proto_tree_add_boolean_hidden(tree, hf_beep_negative, tvb, offset, 1, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_negative, tvb, offset, 1, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
       proto_tree_add_text(tree, tvb, offset, 1, "Status: Negative");
     }
 
@@ -309,6 +316,7 @@ static int num_len(tvbuff_t *tvb, int offset)
 static int
 check_term(tvbuff_t *tvb, int offset, proto_tree *tree)
 {
+	proto_item  *hidden_item;
 
   /* First, check for CRLF, or, if global_beep_strict_term is false,
    * one of CR or LF ... If neither of these hold, we add an element
@@ -331,7 +339,8 @@ check_term(tvbuff_t *tvb, int offset, proto_tree *tree)
 
     if (tree) {
       proto_tree_add_text(tree, tvb, offset, 1, "Nonstandard Terminator: CR");
-      proto_tree_add_boolean_hidden(tree, hf_beep_proto_viol, tvb, offset, 1, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_proto_viol, tvb, offset, 1, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
     }
     return 1;
 
@@ -340,7 +349,8 @@ check_term(tvbuff_t *tvb, int offset, proto_tree *tree)
 
     if (tree) {
       proto_tree_add_text(tree, tvb, offset, 1, "Nonstandard Terminator: LF");
-      proto_tree_add_boolean_hidden(tree, hf_beep_proto_viol, tvb, offset, 1, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_proto_viol, tvb, offset, 1, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
     }
     return 1;
 
@@ -349,7 +359,8 @@ check_term(tvbuff_t *tvb, int offset, proto_tree *tree)
 
     if (tree) {
       proto_tree_add_text(tree, tvb, offset, 2, "PROTOCOL VIOLATION, Invalid Terminator: %s", tvb_format_text(tvb, offset, 2));
-      proto_tree_add_boolean_hidden(tree, hf_beep_proto_viol, tvb, offset, 2, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_proto_viol, tvb, offset, 2, TRUE);
+	  PROTO_ITEM_SET_HIDDEN(hidden_item);
     }
     return -1;
 
@@ -385,13 +396,13 @@ static int header_len(tvbuff_t *tvb, int offset)
 
 static int
 dissect_beep_mime_header(tvbuff_t *tvb, int offset,
-			 struct beep_proto_data *frame_data,
+			 struct beep_proto_data *beep_frame_data,
 			 proto_tree *tree)
 {
   proto_tree    *ti = NULL, *mime_tree = NULL;
   int           mime_length = header_len(tvb, offset), cc = 0;
 
-  if (frame_data && !frame_data->mime_hdr) return 0;
+  if (beep_frame_data && !beep_frame_data->mime_hdr) return 0;
 
   if (tree) {
 
@@ -437,6 +448,7 @@ static int
 dissect_beep_int(tvbuff_t *tvb, int offset,
 		    proto_tree *tree, int hf, int *val, int *hfa[])
 {
+  proto_item  *hidden_item;
   int ival, ind = 0;
   unsigned int i = num_len(tvb, offset);
   guint8 int_buff[100];
@@ -455,7 +467,8 @@ dissect_beep_int(tvbuff_t *tvb, int offset,
 
   while (hfa[ind]) {
 
-    proto_tree_add_uint_hidden(tree, *hfa[ind], tvb, offset, i, ival);
+    hidden_item = proto_tree_add_uint(tree, *hfa[ind], tvb, offset, i, ival);
+	PROTO_ITEM_SET_HIDDEN(hidden_item);
     ind++;
 
   }
@@ -468,7 +481,7 @@ dissect_beep_int(tvbuff_t *tvb, int offset,
 
 static void
 set_mime_hdr_flags(int more, struct beep_request_val *request_val,
-		   struct beep_proto_data *frame_data, packet_info *pinfo)
+		   struct beep_proto_data *beep_frame_data, packet_info *pinfo)
 {
 
   if (!request_val) return; /* Nothing to do ??? */
@@ -477,14 +490,14 @@ set_mime_hdr_flags(int more, struct beep_request_val *request_val,
 
     if (request_val->c_mime_hdr) {
 
-      frame_data->mime_hdr = 0;
+      beep_frame_data->mime_hdr = 0;
 
       if (!more) request_val->c_mime_hdr = 0;
 
     }
     else {
 
-      frame_data->mime_hdr = 1;
+      beep_frame_data->mime_hdr = 1;
 
       if (more) request_val->c_mime_hdr = 1;
 
@@ -495,14 +508,14 @@ set_mime_hdr_flags(int more, struct beep_request_val *request_val,
 
     if (request_val->s_mime_hdr) {
 
-      frame_data->mime_hdr = 0;
+      beep_frame_data->mime_hdr = 0;
 
       if (!more) request_val->s_mime_hdr = 0;
 
     }
     else {
 
-      frame_data->mime_hdr = 1;
+      beep_frame_data->mime_hdr = 1;
 
       if (more) request_val->s_mime_hdr = 1;
 
@@ -523,9 +536,10 @@ set_mime_hdr_flags(int more, struct beep_request_val *request_val,
 static int
 dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		  proto_tree *tree, struct beep_request_val *request_val,
-		  struct beep_proto_data *frame_data)
+		  struct beep_proto_data *beep_frame_data)
 {
   proto_tree     *ti = NULL, *hdr = NULL;
+  proto_item	 *hidden_item;
   int            st_offset, msgno, ansno, seqno, size, channel, ackno, window, cc,
                  more;
 
@@ -553,7 +567,7 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
       hdr = proto_item_add_subtree(ti, ett_header);
 
-      proto_tree_add_boolean_hidden(hdr, hf_beep_req, tvb, offset, 3, TRUE);
+      hidden_item = proto_tree_add_boolean(hdr, hf_beep_req, tvb, offset, 3, TRUE);
       proto_tree_add_text(hdr, tvb, offset, 3, cmd_temp);
     }
 
@@ -570,10 +584,10 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
     /* Insert the more elements ... */
     if ((more = dissect_beep_more(tvb, offset, hdr)) >= 0) {
       /* Figure out which direction this is in and what mime_hdr flag to
-       * add to the frame_data. If there are missing segments, this code
+       * add to the beep_frame_data. If there are missing segments, this code
        * will get it wrong!
        */
-      set_mime_hdr_flags(more, request_val, frame_data, pinfo);
+      set_mime_hdr_flags(more, request_val, beep_frame_data, pinfo);
     }
     else {  /* Protocol violation, so dissect rest as undisectable */
       if (tree) {
@@ -599,8 +613,8 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
     if (request_val)   /* FIXME, is this the right order ... */
       request_val -> size = size;  /* Stash this away */
     else {
-      frame_data->pl_size = size;
-      if (frame_data->pl_size < 0) frame_data->pl_size = 0; /* FIXME: OK? */
+      beep_frame_data->pl_size = size;
+      if (beep_frame_data->pl_size < 0) beep_frame_data->pl_size = 0; /* FIXME: OK? */
     }
     /* offset += 1; skip the space */
 
@@ -632,8 +646,8 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
     /* Insert MIME header ... */
 
-    if (frame_data && frame_data->mime_hdr)
-      offset += dissect_beep_mime_header(tvb, offset, frame_data, hdr);
+    if (beep_frame_data && beep_frame_data->mime_hdr)
+      offset += dissect_beep_mime_header(tvb, offset, beep_frame_data, hdr);
 
     /* Now for the payload, if any */
 
@@ -660,20 +674,20 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	if (request_val->size < 0) request_val->size = 0;
       }
       else {
-	frame_data->pl_size -= pl_size;
-	if (frame_data->pl_size < 0) frame_data->pl_size = 0;
+	beep_frame_data->pl_size -= pl_size;
+	if (beep_frame_data->pl_size < 0) beep_frame_data->pl_size = 0;
       }
     }
 
     /* If anything else left, dissect it ... */
 
     if (tvb_length_remaining(tvb, offset) > 0)
-      offset += dissect_beep_tree(tvb, offset, pinfo, tree, request_val, frame_data);
+      offset += dissect_beep_tree(tvb, offset, pinfo, tree, request_val, beep_frame_data);
 
   } else if (tvb_strneql(tvb, offset, "SEQ ", 4) == 0) {
 
     if (tree) {
-      proto_tree_add_boolean_hidden(tree, hf_beep_seq, tvb, offset, 3, TRUE);
+      hidden_item = proto_tree_add_boolean(tree, hf_beep_seq, tvb, offset, 3, TRUE);
       proto_tree_add_text(tree, tvb, offset, 3, "Command: SEQ");
     }
 
@@ -726,7 +740,7 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
       tr = proto_item_add_subtree(ti, ett_trailer);
 
-      proto_tree_add_boolean_hidden(tr, hf_beep_end, tvb, offset, 3, TRUE);
+      hidden_item = proto_tree_add_boolean(tr, hf_beep_end, tvb, offset, 3, TRUE);
       proto_tree_add_text(tr, tvb, offset, 3, "Command: END");
 
     }
@@ -768,8 +782,8 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
       }
 
-    } else if (frame_data) {
-      pl_size = MIN(frame_data->pl_size, tvb_length_remaining(tvb, offset));
+    } else if (beep_frame_data) {
+      pl_size = MIN(beep_frame_data->pl_size, tvb_length_remaining(tvb, offset));
     } else { /* Just in case */
       pl_size = tvb_length_remaining(tvb, offset);
     }
@@ -800,13 +814,13 @@ dissect_beep_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	if (request_val->size < 0) request_val->size = 0;
       }
       else {
-	frame_data->pl_size -= pl_size;
-	if (frame_data->pl_size < 0) frame_data->pl_size = 0;
+	beep_frame_data->pl_size -= pl_size;
+	if (beep_frame_data->pl_size < 0) beep_frame_data->pl_size = 0;
       }
     }
 
     if (tvb_length_remaining(tvb, offset) > 0) {
-      offset += dissect_beep_tree(tvb, offset, pinfo, tree, request_val, frame_data);
+      offset += dissect_beep_tree(tvb, offset, pinfo, tree, request_val, beep_frame_data);
     }
   }
 
@@ -818,7 +832,7 @@ static void
 dissect_beep(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
   int offset;
-  struct beep_proto_data  *frame_data = NULL;
+  struct beep_proto_data  *beep_frame_data = NULL;
   proto_tree              *beep_tree = NULL, *ti = NULL;
   conversation_t          *conversation = NULL;
   struct beep_request_key request_key, *new_request_key;
@@ -848,9 +862,9 @@ dissect_beep(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
    * info first.
    */
 
-  frame_data = p_get_proto_data(pinfo->fd, proto_beep);
+  beep_frame_data = p_get_proto_data(pinfo->fd, proto_beep);
 
-  if (!frame_data) {
+  if (!beep_frame_data) {
 
     conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype,
 				       pinfo->srcport, pinfo->destport, 0);
@@ -922,9 +936,9 @@ dissect_beep(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
    * We also depend on the first frame in a group having a pl_size of 0.
    */
 
-  if (frame_data && frame_data->pl_left > 0) {
+  if (beep_frame_data && beep_frame_data->pl_left > 0) {
 
-    int pl_left = frame_data->pl_left;
+    int pl_left = beep_frame_data->pl_left;
 
     pl_left = MIN(pl_left, tvb_length_remaining(tvb, offset));
 
@@ -945,13 +959,13 @@ dissect_beep(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      * elsewhere for other frames
      */
 
-    frame_data = se_alloc(sizeof(struct beep_proto_data));
+    beep_frame_data = se_alloc(sizeof(struct beep_proto_data));
 
-    frame_data->pl_left = pl_left;
-    frame_data->pl_size = 0;
-    frame_data->mime_hdr = 0;
+    beep_frame_data->pl_left = pl_left;
+    beep_frame_data->pl_size = 0;
+    beep_frame_data->mime_hdr = 0;
 
-    p_add_proto_data(pinfo->fd, proto_beep, frame_data);
+    p_add_proto_data(pinfo->fd, proto_beep, beep_frame_data);
 
   }
 
@@ -959,21 +973,21 @@ dissect_beep(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
    * This _must_ come after the checks above ...
    */
 
-  if (frame_data == NULL) {
+  if (beep_frame_data == NULL) {
 
-    frame_data = se_alloc(sizeof(struct beep_proto_data));
+    beep_frame_data = se_alloc(sizeof(struct beep_proto_data));
 
-    frame_data->pl_left = 0;
-    frame_data->pl_size = 0;
-    frame_data->mime_hdr = 0;
+    beep_frame_data->pl_left = 0;
+    beep_frame_data->pl_size = 0;
+    beep_frame_data->mime_hdr = 0;
 
-    p_add_proto_data(pinfo->fd, proto_beep, frame_data);
+    p_add_proto_data(pinfo->fd, proto_beep, beep_frame_data);
 
   }
 
   if (tvb_length_remaining(tvb, offset) > 0) {
 
-    offset += dissect_beep_tree(tvb, offset, pinfo, beep_tree, request_val, frame_data);
+    offset += dissect_beep_tree(tvb, offset, pinfo, beep_tree, request_val, beep_frame_data);
 
   }
 
@@ -1078,7 +1092,7 @@ proto_register_beep(void)
 void
 proto_reg_handoff_beep(void)
 {
-  static int beep_prefs_initialized = FALSE;
+  static gboolean beep_prefs_initialized = FALSE;
   static dissector_handle_t beep_handle;
 
   if (!beep_prefs_initialized) {
