@@ -2,7 +2,7 @@
  * Routines for bittorrent packet dissection
  * Copyright (C) 2004 Jelmer Vernooij <jelmer@samba.org>
  *
- * $Id: packet-bittorrent.c 19751 2006-10-31 09:29:07Z sahlberg $
+ * $Id: packet-bittorrent.c 26091 2008-08-26 01:28:50Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -37,7 +37,6 @@
 #include <epan/conversation.h>
 #include <epan/packet.h>
 #include <epan/strutil.h>
-#include <epan/emem.h>
 
 #include "packet-tcp.h"
 
@@ -61,11 +60,11 @@
 
 #define BITTORRENT_HEADER_LENGTH          4
 
-/* 
+/*
  * Azureus messages are specified by name so these are made up numbers
  * for internal identification only.
  *
- * Standard BT message types are a single byte, so these won't clash 
+ * Standard BT message types are a single byte, so these won't clash
  */
 #define AZUREUS_MESSAGE_HANDSHAKE         256
 #define AZUREUS_MESSAGE_KEEP_ALIVE        257
@@ -103,7 +102,7 @@ static const value_string azureus_priorities[] = {
 
 
 struct amp_message {
-  char *name;
+  const char *name;
   guint32 value;
 };
 
@@ -170,7 +169,7 @@ static gboolean decode_client_information = FALSE;
 
 struct client_information {
    char id[4];
-   char *name;
+   const char *name;
 };
 
 static struct client_information peer_id[] = {
@@ -199,7 +198,7 @@ static guint get_bittorrent_pdu_length(packet_info *pinfo _U_, tvbuff_t *tvb, in
    guint32 length;
 
    if (tvb_get_guint8(tvb, offset) == 19 &&
-      tvb_memeql(tvb, offset + 1, "BitTorrent protocol", 19) == 0) {
+       tvb_memeql(tvb, offset + 1, "BitTorrent protocol", 19) == 0) {
       /* Return the length of a Handshake message */
       return 1 + /* pstrlen */
          19 +    /* pstr */
@@ -278,7 +277,7 @@ static int dissect_bencoding_str(tvbuff_t *tvb, packet_info *pinfo _U_,
 	}
 	if (treeadd==2) {
 	  proto_item_append_text(ti, "  Value: %s", format_text(ep_tvb_memdup(tvb, offset+used, stringlen), stringlen));
-	}	  
+	}
       }
       return used+stringlen;
     }
@@ -338,7 +337,7 @@ static int dissect_bencoding_int(tvbuff_t *tvb, packet_info *pinfo _U_,
 	proto_tree_add_int(tree, hf_bittorrent_bint, tvb, offset, used, ival);
 	if (treeadd==2) {
 	  proto_item_append_text(ti, "  Value: %d", ival);
-	}	  
+	}
       }
       return used;
 
@@ -433,7 +432,7 @@ static int dissect_bencoding_rec(tvbuff_t *tvb, packet_info *pinfo _U_,
 	}
 	return op2len;
       }
-      
+
       if (dtree) {
 	ti = proto_tree_add_item(dtree, hf_bittorrent_bdict_entry, tvb, offset+used, op1len+op2len, FALSE);
 	itree = proto_item_add_subtree(ti, ett_bittorrent_bdict_entry);
@@ -441,7 +440,7 @@ static int dissect_bencoding_rec(tvbuff_t *tvb, packet_info *pinfo _U_,
 	dissect_bencoding_str(tvb, pinfo, offset+used, length, itree, ti, 1);
 	dissect_bencoding_rec(tvb, pinfo, offset+used+op1len, length-op1len, itree, level+1, ti, 2);
       }
-      
+
       used += op1len+op2len;
       length -= op1len+op2len;
     }
@@ -465,7 +464,7 @@ static int dissect_bencoding_rec(tvbuff_t *tvb, packet_info *pinfo _U_,
       if (op=='e') {
 	return used+1;
       }
-     
+
       oplen = dissect_bencoding_rec(tvb, pinfo, offset+used, length, itree, level+1, ti, 0);
       if (oplen<1) return oplen;
 
@@ -488,12 +487,8 @@ static int dissect_bencoding_rec(tvbuff_t *tvb, packet_info *pinfo _U_,
     if (tree) {
       proto_tree_add_text(tree, tvb, offset, -1, "Decode Aborted: Invalid Bencoding");
     }
-    return -1;
   }
 
-  if (tree) {
-    proto_tree_add_text(tree, tvb, offset, -1, "Decode Aborted: Internal Error");
-  }
   return -1;
 }
 
@@ -502,7 +497,7 @@ static void dissect_bencoding(tvbuff_t *tvb, packet_info *pinfo _U_,
 {
   dissect_bencoding_rec(tvb, pinfo, offset, length, tree, 0, NULL, 0);
 }
-    
+
 static void dissect_bittorrent_message (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
    int offset = 0;
@@ -525,7 +520,7 @@ static void dissect_bittorrent_message (tvbuff_t *tvb, packet_info *pinfo, proto
       type = tvb_get_guint8(tvb, offset + BITTORRENT_HEADER_LENGTH);
 
       if (type==BITTORRENT_MESSAGE_CHOKE && length>4) {
-	/* 
+	/*
 	 * Choke messages have no payload, so this is likely an Azureus
 	 * Messaging Protocol packet
 	 */
@@ -559,7 +554,7 @@ static void dissect_bittorrent_message (tvbuff_t *tvb, packet_info *pinfo, proto
 	msgtype = match_strval(type, azureus_messages);
 	} */
       if (msgtype == NULL) {
-         proto_tree_add_text(tree, tvb, offset, -1, "Continuation data"); 
+         proto_tree_add_text(tree, tvb, offset, -1, "Continuation data");
          if (check_col(pinfo->cinfo, COL_INFO)) {
             col_set_str(pinfo->cinfo, COL_INFO, "Continuation data");
          }
@@ -640,7 +635,7 @@ static void dissect_bittorrent_message (tvbuff_t *tvb, packet_info *pinfo, proto
       break;
 
    case BITTORRENT_MESSAGE_BITFIELD:
-      proto_tree_add_item(mtree, hf_bittorrent_bitfield_data, tvb, offset, length, FALSE); 
+      proto_tree_add_item(mtree, hf_bittorrent_bitfield_data, tvb, offset, length, FALSE);
       proto_item_append_text(ti, ", Len:0x%x", length);
       if (check_col(pinfo->cinfo, COL_INFO)) {
          col_append_fstr(pinfo->cinfo, COL_INFO, ", Len:0x%x", length);
@@ -690,15 +685,15 @@ static void dissect_bittorrent_welcome (tvbuff_t *tvb, packet_info *pinfo _U_, p
    int offset = 0;
    int i;
    char *version;
-   
+
    if (check_col(pinfo->cinfo, COL_INFO)) {
       col_set_str(pinfo->cinfo, COL_INFO, "Handshake");
    }
-   
+
    proto_tree_add_item(tree, hf_bittorrent_prot_name_len, tvb, offset, 1, FALSE); offset+=1;
    proto_tree_add_item(tree, hf_bittorrent_prot_name, tvb, offset, 19, FALSE); offset += 19;
    proto_tree_add_item(tree, hf_bittorrent_reserved, tvb, offset, 8, FALSE); offset += 8;
-   
+
    proto_tree_add_item(tree, hf_bittorrent_sha1_hash, tvb, offset, 20, FALSE);
    offset += 20;
 
@@ -706,16 +701,15 @@ static void dissect_bittorrent_welcome (tvbuff_t *tvb, packet_info *pinfo _U_, p
    if(decode_client_information) {
       for(i = 0; peer_id[i].id[0] != '\0'; ++i)
       {
-         if(tvb_memeql(tvb, offset, peer_id[i].id, strlen(peer_id[i].id)) == 0) {
+	 if(tvb_memeql(tvb, offset, peer_id[i].id, strlen(peer_id[i].id)) == 0) {
             /* The version number is 4 numeric characters for the
                client ids beginning with '-' and 3 characters for the
                rest. */
-            version = tvb_get_string(tvb, offset + strlen(peer_id[i].id),
-               (peer_id[i].id[0] == '-') ? 4 : 3);
+	    version = tvb_get_ephemeral_string(tvb, offset + strlen(peer_id[i].id),
+	       (peer_id[i].id[0] == '-') ? 4 : 3);
             proto_tree_add_text(tree, tvb, offset, 20, "Client is %s v%s",
                peer_id[i].name,
-               format_text(version, (peer_id[i].id[0] == '-') ? 4 : 3));
-            g_free(version);
+	       format_text((guchar*)version, (peer_id[i].id[0] == '-') ? 4 : 3));
             break;
          }
       }
@@ -726,20 +720,20 @@ static void dissect_bittorrent_welcome (tvbuff_t *tvb, packet_info *pinfo _U_, p
 static void dissect_bittorrent_tcp_pdu (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
    proto_item *ti;
-   
+
    if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
       col_set_str(pinfo->cinfo, COL_PROTOCOL, "BitTorrent");
    }
-   
+
    if (check_col(pinfo->cinfo, COL_INFO)) {
       col_set_str(pinfo->cinfo, COL_INFO, "BitTorrent ");
    }
-   
-   ti = proto_tree_add_item (tree, proto_bittorrent, tvb, 0, -1, FALSE);   
+
+   ti = proto_tree_add_item (tree, proto_bittorrent, tvb, 0, -1, FALSE);
    tree = proto_item_add_subtree(ti, ett_bittorrent);
-   
+
    if (tvb_get_guint8(tvb, 0) == 19 &&
-      tvb_memeql(tvb, 1, "BitTorrent protocol", 19) == 0) {
+       tvb_memeql(tvb, 1, "BitTorrent protocol", 19) == 0) {
       dissect_bittorrent_welcome(tvb, pinfo, tree);
    } else {
       dissect_bittorrent_message(tvb, pinfo, tree);
@@ -762,9 +756,10 @@ static gboolean test_bittorrent_packet (tvbuff_t *tvb, packet_info *pinfo,
 {
    conversation_t *conversation;
 
-   if (tvb_bytes_exist(tvb, 0, 20) &&
+   if (tvb_length(tvb) >= 20 &&
        tvb_get_guint8(tvb, 0) == 19 &&
        tvb_memeql(tvb, 1, "BitTorrent protocol", 19) == 0) {
+      /* XXXX why new ? */
       conversation = conversation_new (pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
 
       conversation_set_dissector(conversation, dissector_handle);
@@ -781,7 +776,7 @@ void
 proto_register_bittorrent(void)
 {
    static hf_register_info hf[] = {
-      { &hf_bittorrent_field_length, 
+      { &hf_bittorrent_field_length,
       { "Field Length", "bittorrent.length", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }
       },
       { &hf_bittorrent_prot_name_len,
@@ -799,12 +794,12 @@ proto_register_bittorrent(void)
       { &hf_bittorrent_peer_id,
       { "Peer ID", "bittorrent.peer_id", FT_BYTES, BASE_DEC, NULL, 0x0, "", HFILL }
       },
-      { &hf_bittorrent_msg, 
+      { &hf_bittorrent_msg,
       { "Message", "bittorrent.msg", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }
       },
       { &hf_bittorrent_msg_len,
       { "Message Length", "bittorrent.msg.length", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }
-      }, 
+      },
       { &hf_bittorrent_msg_type,
       { "Message Type", "bittorrent.msg.type", FT_UINT8, BASE_DEC, VALS(bittorrent_messages), 0x0, "", HFILL }
       },
@@ -829,7 +824,7 @@ proto_register_bittorrent(void)
       { &hf_bittorrent_piece_begin,
       { "Begin offset of piece", "bittorrent.piece.begin", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }
       },
-      { &hf_bittorrent_piece_data, 
+      { &hf_bittorrent_piece_data,
       { "Data in a piece", "bittorrent.piece.data", FT_BYTES, BASE_HEX, NULL, 0x0, "", HFILL }
       },
       { &hf_bittorrent_piece_length,
